const Query = require('../service/query');
const Util = require('../service/util');
const Crypt = require('../service/crypt');
const { format } = require('@fast-csv/format');

const route = (router) => {
  router.post('/csv/sales', async function(req, res, next) {
    try {
      const header = req.headers;

      if (!('authorization' in header)) {
        throw new Error(`Missing authorization in header request`);
      }

      const authorization = header.authorization;

      const token = Util.isValidBearerAuth(authorization);
      const { DB_CRYPT_KEY } = process.env;

      const connParams = JSON.parse(Crypt.decrypt(token, DB_CRYPT_KEY));

      const body = req.body; 
      
      if (!('startDate' in body)) {
        throw new Error(`Missing 'startDate' in body request params.`);
      }

      Util.dateValidation(body.startDate, 'startDate');
      Util.startDateValidation(body.startDate);

      if ('endDate' in body) {
        Util.dateValidation(body.endDate, 'endDate');
        Util.endDateValidation(body.endDate);
      } else {
        const today = new Date();
        const yesterday = new Date(today);
        yesterday.setDate(today.getDate() - 1);

        body.endDate = Util.convertDate(yesterday, 'dd/mm/yyyy');
      }

      Util.startDateBeforeEndDate(body.startDate, body.endDate);
      Util.dateRangeValidation(body.startDate, body.endDate, process.env.MAX_INTERVAL_CSV);

      const params = {
        "P_DTINICIAL": body.startDate,
        "P_DTFINAL": body.endDate
      };

      const sql = Query.get('getSales');
      const data = await Query.run(connParams, sql, params);

      const csvStream = format({ headers: true });
      
      const startDateReplace = params.P_DTINICIAL.replace('/ ', '-');
      const endDateReplace = params.P_DTFINAL.replace('/ ', '-');

      // Definindo o nome do arquivo de saída
      res.setHeader('Content-Disposition', `attachment; filename="sales-${startDateReplace}-to-${endDateReplace}-from-${connParams.dbname}.csv"`);
      res.setHeader('Content-Type', 'text/csv');

      // Streaming do CSV para a resposta
      csvStream.pipe(res).on('end', () => process.exit());
      
      // Exemplo de dados para o CSV
      data.rows.forEach(row => csvStream.write(row)); // Adiciona cada objeto do array como uma linha no CSV
      
      csvStream.end();
    } catch (error) {
      res.json({
        "error": true,
        "message": error.message
      });
    }
  });
}

module.exports = route;
