const oracledb = require('oracledb');

class Oracle {
  constructor() {
    this.oracledb = oracledb;
    this.connection = {};

    const libDir = process.env.LD_LIBRARY_PATH
    if (libDir) oracledb.initOracleClient({ libDir })
  }
  

  async connect(connParams) {
    try {
      const { LOCAL_MODE } = process.env;
      let conn;

      if (LOCAL_MODE === 'YES') {
        const { 
          DB_USER: user,
          DB_PASSWORD: password,
          DB_CONN_STR: connectString
        } = process.env;

        conn = { user, password, connectString };
      } else {
        conn = { externalAuth: true, connectString: connParams.dbname};
      }

      this.connection = await oracledb.getConnection(conn);
      return this.connection;
    } catch (error) {
      // DataBaseLogger.write(error, {type: 'CONNECTION'});
      return Promise.reject(new Error('DB_ERROR'));
    }
  }

  async fetch(sqlString, params) {
    try {
      let sqlPrepare = sqlString;

      if (params) {
        Object.keys(params).forEach((key) => {
          const replace = new RegExp(`:${key}`, 'g');
          sqlPrepare = sqlPrepare.replace(replace, `'${params[key]}'`);
        });
      }

      if (! await this.checkConnection()) {
        this.connection = await this.connect();
        await this.connection.execute("ALTER SESSION SET TIME_ZONE='UTC'");
      }
      
      return this.connection.execute(sqlPrepare);
    } catch (error) {
      // DataBaseLogger.write(error, {type: 'QUERY_EXECUTION', sql: sqlString, params})
      return Promise.reject(new Error('DB_ERROR'));
    }
  }

  async fetchAssoc(sqlString, params, executionParams = {}) {
    try {
      let sqlPrepare = sqlString;
      if (params) {
        Object.keys(params).forEach((key) => {
          const replace = new RegExp(`:${key}`, 'g');
          sqlPrepare = sqlPrepare.replace(replace, `'${params[key]}'`);
        });
      }    
    
      if (! await this.checkConnection()) {
        this.connection = await this.connect();

        await this.connection.execute("ALTER SESSION SET TIME_ZONE='UTC'");
      }
      
      oracledb.fetchAsString = [ oracledb.CLOB ];
      const oracleParams = {
        ...executionParams,
        outFormat: oracledb.OUT_FORMAT_OBJECT
      }
      const result = await this.connection.execute(
        sqlPrepare,
        [],
        oracleParams
      );
      return result;
    } catch (error) {
      // DataBaseLogger.write(error, {type: 'QUERY_EXECUTION', sql: sqlString, params})
      return Promise.reject(new Error('DB_ERROR'));
    }
  }

  async close() {
    await this.connection.close(function (err) {
      if (err) {
        SystemLogger.error(err);
      }
    });
  }

  async commit() {
    if(await this.checkConnection()) {
      await this.connection.commit();
    }
  }

  async rollback() {
    if (await this.checkConnection()) {
      await this.connection.rollback();
    }
  }

  async checkConnection() {
    if(!this.connection) return false;
    if (Object.keys(this.connection).length === 0) {
      return false;
    }

    if (typeof this.connection.ping !== 'function') {
      return false;
    }

    return this.connection.ping().then((err) => {
      if(err) {
        this.connection.close();
        return false;
      }
      return true;
    }).catch( () => {
      return false;
    });
  }

  getParamType(type) {
    const types = {
      'number': oracledb.NUMBER,
      'string': oracledb.STRING,
      'cursor': oracledb.CURSOR
    }

    return types[type];
  }

  createReturnBinds(outParams, sqlPrepare) {
    const sqlParams = outParams.map((p) => p.name).join(', ');
    const sqlParamsInto = outParams.map((p) => `:${p.name}`).join(', ');
    const sqlReturn = `RETURNING ${sqlParams} INTO ${sqlParamsInto}`;
    const query = sqlPrepare + sqlReturn;

    let bind = {};
    
    for (const param of outParams) {
      bind[param.name] = { type: this.getParamType(param.type), dir: oracledb.BIND_OUT }
    }

    return { query, bind };
  }

  async execute(sqlString, params, outParams = [], connection = null) {
    let sqlPrepare = sqlString;
    let paramsKeys = params ? Object.keys(params) : null;
    let anonymousBlock = false;
    let colPrepare = {};

    paramsKeys = params
      ? paramsKeys.sort(function (a1, a2) {
          return ~(a1.length < a2.length);
        })
      : [];

    if (paramsKeys) {
      paramsKeys.forEach((key) => {
        try {
          const replace = new RegExp(`:${key}(\\s|,|\\)|$)`, 'g');
          const comma = sqlPrepare.match(replace)[0].includes(',') ? ',' : '';
          const closeParenthesis = sqlPrepare.match(replace)[0].includes(')') ? ')' : '';

          if (params[key] == null) {
            sqlPrepare = sqlPrepare.replace(replace, `null${comma}${closeParenthesis}`);
          } else if (typeof params[key] === 'object') {
            anonymousBlock = true;
            const colName = key;

            const regExp = new RegExp('"', 'g');
            let paramData = String.fromCharCode(...params[key].data);
            paramData = paramData.replace(regExp, "'");
            colPrepare[colName] = paramData;
          } else if (typeof params[key] === 'string') {
            const regExp = new RegExp("'", 'g');
            params[key] = params[key].replace(regExp, "''");
            sqlPrepare = sqlPrepare.replace(replace, `'${params[key]}'${comma}${closeParenthesis}`);
          } else {
            sqlPrepare = sqlPrepare.replace(replace, `${params[key]}${comma}${closeParenthesis}`);
          }
        } catch (error) {
          // erros que podem acontecer é a chave aparecer no update não como condição e sim para substituir no update
        }
      });
    }
    await this.connect();

    sqlPrepare = sqlPrepare.replace(/TIMESTAMP null/gi, 'null');
    sqlPrepare = sqlPrepare.replace(/TIMESTAMP 'Invalid date'/gi, 'null');

    const dbConnection = connection || this.connection;
    
    if (anonymousBlock) {
      const connect = dbConnection
        .execute(`BEGIN ${sqlPrepare}; END;`, colPrepare)
        .then((result) => {
          const newResult = result;
          newResult.connection = this.connection;

          return newResult;
        })
        .catch((error) => {
          // DataBaseLogger.write(error, {type: 'ANONYMOUS_BLOCK', sql: sqlString, params});
          return Promise.reject(new Error('DB_ERROR'));
        });
      return connect;
    }
    try {
      oracledb.fetchAsString = [ oracledb.CLOB ];
      let bindParams = {};

      if (outParams.length) {
        const { query, bind } = this.createReturnBinds(outParams, sqlPrepare);
  
        sqlPrepare = query;
        bindParams = bind;
      }
      
      const dbConnectionResult = await dbConnection.execute(sqlPrepare, bindParams, 
        { 
          outFormat: oracledb.OUT_FORMAT_OBJECT 
        }
      );

      dbConnectionResult.connection = dbConnection;

      return dbConnectionResult;
    } catch (error) {
      // DataBaseLogger.write(error, {sql: sqlString, params, type: 'SQL_EXECUTION'});
      throw error;
    }
  }

  // getConnectionParams() {
  //   const dbName = process.argv[2] || '';
  //   const { 
  //     DB_USER: user,
  //     DB_PASSWORD: password,
  //     DB_CONN_STR: connectString
  //    } = process.env;

  //    if(["EXTERNO", "DEV"].includes(dbName)) {
  //     return { user, password, connectString };
  //    }
  //    return { externalAuth: true, connectString: dbName};
  // }

  // async getNewConnection() {
  //   const connParams = this.getConnectionParams();
  //   return await oracledb.getConnection(connParams);
  // }
}

module.exports = new Oracle();
