const Base64 = require('./base64');
const { createHash } = require('crypto');

class Util {
  dateRangeValidation(startDate, endDate, range) {
    const start = this.parseDate(startDate);
    const end = this.parseDate(endDate);

    const diff = end - start;
    const diffSeconds = diff / 1000;

    const maxIntervalSeconds = this.convertTime(range);

    if (diffSeconds > maxIntervalSeconds) {
      throw new Error(`Date range between 'startDate' (${startDate}) and 'endDate' (${endDate}) cannot be higher than ${range}`);
    }
  }

  dateValidation(date, type) {
    const datePattern = /^(\d{2})\/(\d{2})\/(\d{4})$/;
    const match = date.match(datePattern);

    if (!match) {
      throw new Error(`'${type}' must respect the following format: 'dd/mm/yyyy'`);
    }

    const day = parseInt(match[1], 10);
    const month = parseInt(match[2], 10) - 1;
    const year = parseInt(match[3], 10);

    const dateCheck = new Date(year, month, day);

    if (dateCheck.getDate() !== day || dateCheck.getMonth() !== month || dateCheck.getFullYear() !== year) {
      throw new Error(`'${type}' must be a valid date`);
    }
  }

  startDateValidation(date) {
    const inputDate = this.parseDate(date);

    const currentYear = new Date().getFullYear();
    const firstDay = new Date(currentYear, 0, 1);

    if (firstDay > inputDate) {
      const firstDayFormatted = this.convertDate(firstDay, 'dd/mm/yyyy');
      throw new Error(`'startDate' must be later than ${firstDayFormatted}`);
    }
  }

  endDateValidation(date) {
    const inputDate = this.parseDate(date);

    const today = new Date();
    const yesterday = new Date(today);
    yesterday.setDate(today.getDate() - 1);

    if (inputDate > yesterday) {
      const todayFormmated = this.convertDate(today, 'dd/mm/yyyy');
      throw new Error(`'endDate' must be earlier than today: ${todayFormmated}`);
    }
  }

  startDateBeforeEndDate(startDate, endDate) {
    const start = this.parseDate(startDate);
    const end = this.parseDate(endDate);

    if (start > end) {
      throw new Error(`'startDate' (${startDate}) must be earlier or equal 'endDate' (${endDate})`);
    }
  }

  convertDate(date, format) {
    const day = this.padZero(date.getDate());
    const month = this.padZero(date.getMonth() + 1);
    const year = date.getFullYear();
    const hours = this.padZero(date.getHours());
    const minutes = this.padZero(date.getMinutes());
    const seconds = this.padZero(date.getSeconds());

    const formattedDate = format
      .replace('dd', day)
      .replace('mm', month)
      .replace('yyyy', year)
      .replace('yy', String(year).slice(-2))
      .replace('HH', hours)
      .replace('MM', minutes)
      .replace('ss', seconds);

    return formattedDate;
  }

  convertTime(time) {
    const timeValue = parseInt(time, 10); // Get the numeric part
    const timeUnit = time.match(/[a-zA-Z]+/)[0]; // Get the unit part

    let seconds = 0;

    switch(timeUnit) {
      case 'd': // days to seconds
        seconds = timeValue * 86400;
      break;
      
      case 'w': // weeks to seconds
        seconds = timeValue * 604800;
      break;
      
      case 'm': // months to seconds (approximated as 30 days)
        seconds = timeValue * 2592000;
      break;
      
      default:
        throw new Error(`Invalid time unit: Check .env variable 'MAX_INTERVAL'`);
    }

    return seconds;
  }

  parseDate(date) {
    const [day, month, year] = date.split('/').map(Number);
    return new Date(year, month - 1, day);
  }

  padZero(number) {
    return number < 10 ? '0' + number : number;
  }

  isValidBasicAuth(authorization) {
    if (typeof authorization !== 'string' || !authorization.startsWith('Basic')) {
      throw new Error('Invalid Basic authorization');
    }
    
    const credential = Base64.decode(authorization.replace('Basic ', ''), false);
    const [username = '', password = ''] = credential?.split(':') || [];
    
    const usernameToVerify = createHash('sha256').update(username).digest('hex');
    const passwordToVerify = createHash('sha256').update(password).digest('hex');

    const { AUTH_USERNAME, AUTH_PASSWORD } = process.env;

    if (usernameToVerify !== AUTH_USERNAME || passwordToVerify !== AUTH_PASSWORD) {
      throw new Error('Invalid login credentials');
    }
  }

  isValidBearerAuth(authorization) {
    if (typeof authorization !== 'string' || !authorization.startsWith('Bearer')) {
      throw new Error('Invalid Bearer authorization');
    }

    const credential = authorization.replace('Bearer ', '').trim();

    if (credential === '') {
      throw new Error('Empty Bearer authorization');
    }

    return credential;
  }
}

module.exports = new Util();